const config = require('../config.json');
const logger = require('./logger.js');
const api = require('./api.js')
const constants = require('./constants.js');
const modep = require('./modep.js');
const http = require('http');

var server;

function start() {
    return new Promise(function (resolve, reject) {
        if (!server) {
            server = http.createServer();
        }
        server.listen(config.server.port, config.server.listen).on('listening', function () {
            logger.debug('server listening on ' + config.server.listen + ':' + config.server.port + '...');
            handleRequests();
        });
    });
}

function handleRequests() {
    server.on('request', function (request, response) {
        request.timestamp = new Date().getTime();
        if (request.url.endsWith('/')) {
            request.url = request.url.substring(0, request.url.length - 1);
        }
        var endpoint = api.getEndpoints().get(request.url);
        if (!endpoint) {
            endRequest(
                request,
                response,
                'endpoint \'' + request.url + '\' not defined',
                501
            );
            return;
        }
        endpoint = endpoint[request.method];
        if (endpoint == undefined || endpoint.method == undefined) {
            endRequest(
                request,
                response,
                'error: endpoint \'' + request.url + '\' does not have any handlers for ' + request.method + ' requests',
                405
            );
            return;
        }
        getRequestParams(request)
            .then(function (params) {
                return endpoint.method(endpoint.id, params);
            })
            .then(function (result) {
                endRequest(request, response, result);
            })
            .catch(function (err, code) {
                if (code == undefined) {
                    code = 500;
                }
                endRequest(request, response, err, code);
            });
    });
}

function endRequest(request, response, msg, code) {
    if (code == undefined) {
        code = 200;
    }
    var object = {
        request: request,
        code: code
    }
    if (msg != undefined) {
        if (msg instanceof Object) {
            object.data = JSON.stringify(msg);
        } else if (msg instanceof Error) {
            object.data = msg.toString();
        } else {
            object.data = msg;
        }
    }
    response.writeHead(object.code);
    response.end(object.data);
    logger.http(object);
}

function getRequestParams(request) {
    return new Promise(function (resolve, reject) {
        var params = "";
        request.on("data", function (data) {
            params += data;
        });
        request.on("end", function () {
            if (params == undefined || params.length == 0) {
                return resolve();
            }
            return resolve(new URLSearchParams(params));
        });
    });
}

function handlePOSTPedals(request) {
    return new Promise(function (resolve, reject) {
        getRequestParams(request)
            .then(function (params) {
                var pedalId = request.url.substring(request.url.lastIndexOf('/') + 1);
                var controlId = params.get('id');
                if (controlId == undefined) {
                    reject('error: could not handle POST - missing parameter \'id\'', 400)
                }
                var value = params.get('value');
                if (value == undefined) {
                    reject('error: could not handle POST - missing parameter \'value\'', 400)
                }
                modep.getPedalControlById(pedalId, controlId)
                    .then(function (control) {
                        modep.sendValueToControl(value, control);
                    })
                    .then(resolve)
                    .catch(reject);
            })
            .catch(reject);
    });
}

function handlePOSTPedalboards(request) {
    return new Promise(function (resolve, reject) {
        modep.setPedalboardById(request.url.substring(request.url.lastIndexOf('/') + 1))
            .then(resolve)
            .catch(reject);
    });
}

module.exports = {
    start
}